/*
 * Copyright (c) 1996, 1998 Hellmuth Michaelis.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*---------------------------------------------------------------------------*
 *
 *	setnetparm - main.c
 *	-------------------
 *
 *	last edit-date: [Tue Sep  8 15:13:23 1998]
 *
 *	-hm	FreeBSD 2.1.6.1
 *	-hm	menu defaults file
 *	-hm	FreeBSD 2.2.7
 *
 *---------------------------------------------------------------------------*/

#define MAIN

#include "defs.h"

static void finish(void);
static int rd_menudefs(void);
static void hdrline(int row);

#define HELP_IPA "please enter the local IP address as dotted-quad, example: 192.168.168.13"
#define HELP_SNM "please enter the subnet mask as dotted-quad, example: 255.255.255.0"
#define HELP_IFF "please enter the interface flags if necessary, example: -link0 -link1"
#define HELP_HNM "please enter the local hostname string, example: myhostname"
#define HELP_DNM "please enter the domainname string, example: mydomain.org"
#define HELP_GWA "please enter the gateway address as dotted-quad, example: 192.168.168.1"
#define HELP_NSA "please enter the nameserver address as dotted-quad, example: 192.168.168.2"

/*---------------------------------------------------------------------------*
 *	program start
 *---------------------------------------------------------------------------*/
int
main()
{
	FILE *fp;
	int choice = -1;
	int got_defs = 0;
	int dmi = 0;
	int i, j;
	int first = 1;
	
	signal(SIGQUIT, SIG_IGN);
	signal(SIGINT, SIG_IGN);
	signal(SIGHUP, SIG_IGN);
	signal(SIGTERM, SIG_IGN);
	signal(SIGTSTP, SIG_IGN);
	signal(SIGSTOP, SIG_IGN);

	initscr();

	if((fp = fopen(FILE_DEFAULTS, "r")) == NULL)
	{
		mvaddstr(1, (40 - strlen(nodefs) / 2), nodefs);
	}
	else
	{
		char buffer[81] = "";
		
		if(fgets(buffer, 80, fp) != NULL)
			sscanf(buffer, "%s", ipaddr);
		else
			exit(1);

		if(fgets(buffer, 80, fp) != NULL)
			sscanf(buffer, "%s", snmask);
		else
			exit(1);

		if(fgets(buffer, 80, fp) != NULL)
		{
			strcpy(ifflags, buffer);
			ifflags[strlen(buffer)-1] = '\0';
		}
		else
			exit(1);

		if(fgets(buffer, 80, fp) != NULL)
			sscanf(buffer, "%s", hname);
		else
			exit(1);

		if(fgets(buffer, 80, fp) != NULL)
			sscanf(buffer, "%s", dname);
		else
			exit(1);

		if(fgets(buffer, 80, fp) != NULL)
			sscanf(buffer, "%s", gwaddr);
		else
			exit(1);

		if(fgets(buffer, 80, fp) != NULL)
			sscanf(buffer, "%s", nmserver);
		else
			exit(1);


		got_defs = 1;
		fclose(fp);
	}

	hdrline(0);
	
	mvaddstr(POS_DEF, 4, defs);
	mvaddstr(POS_DU,  4, defsu);
	mvprintw(POS_IP,  4, "IP address:  %s", ipaddr);
	mvprintw(POS_SN,  4, "Subnetmask:  %s", snmask);
	mvprintw(POS_FL,  4, "     Flags:  %s", ifflags);	
	mvprintw(POS_HN,  4, "  Hostname:  %s", hname);
	mvprintw(POS_DN,  4, "Domainname:  %s", dname);	
	mvprintw(POS_GW,  4, "   Gateway:  %s", gwaddr);
	mvprintw(POS_NS,  4, "Nameserver:  %s", nmserver);

	dmi = rd_menudefs();

	mainmenu[0] = mitem0;
	mainmenu[1] = mitem1;
	mainmenu[2] = mitem2;

	for(i=3, j=0; ((i < (TABLEN+TABFIX)) && (j < dmi)); i++, j++)
		mainmenu[i] = m_menuitem[j];
		
	mainmenu[i] = NULL;

	
	for (;;)
	{
		refresh();

		if(first && keytimeout > 0)
		{
			first = 0;
			choice = menubar(M_LINE, M_COL, mainmenu, 1, mainmenutitle, 0, keytimeout);
			if(choice == -1)
			{
				finish();
				exit(0);
			}
		}
		else
		{
			if(choice != -1 && choice != ITEM_USE_CURR)
				choice = menubar(M_LINE, M_COL, mainmenu, 1, mainmenutitle, ITEM_USE_NEW-1, 0);
			else
				choice = menubar(M_LINE, M_COL, mainmenu, 1, mainmenutitle, 0, 0);
		}			

		move(20, 0);
		clrtoeol();
		refresh();

		switch(choice)
		{
			case ITEM_USE_CURR:
				if(!got_defs)
				{
					mvaddstr(1, (40 - strlen(nodefs) / 2), nodefs);
					break;
				}
				finish();
				break;

			case ITEM_ENTER:
				curgets(nipaddr, 16, C_LINE, C_COL, ipaddress, 0, ipaddr, HELP_IPA, TYPE_IP);
				mvprintw(POS_IP, POS_COLNEW, "%s", nipaddr);
				clrtoeol();
				refresh();

				curgets(nsnmask, 16, C_LINE, C_COL, subnetmask, 0, snmask, HELP_SNM, TYPE_IP);
				mvprintw(POS_SN, POS_COLNEW, "%s", nsnmask);
				clrtoeol();
				refresh();

				curgets(nifflags, 40, C_LINE, C_COL, ifconfigflags, 0, ifflags, HELP_IFF, TYPE_IFF);
				mvprintw(POS_FL, POS_COLNEW, "%s", nifflags);
				clrtoeol();
				refresh();

				curgets(nhname , 40, C_LINE, C_COL, hostname, 0, hname, HELP_HNM, TYPE_HOST);
				mvprintw(POS_HN, POS_COLNEW, "%s", nhname);
				clrtoeol();
				refresh();

				curgets(ndname , 40, C_LINE, C_COL, domainname, 0, dname, HELP_DNM, TYPE_DOMAIN);
				mvprintw(POS_DN, POS_COLNEW, "%s", ndname);
				clrtoeol();
				refresh();

				curgets(ngwaddr, 16, C_LINE, C_COL, gwaddress, 0, gwaddr, HELP_GWA, TYPE_IP);
				mvprintw(POS_GW, POS_COLNEW, "%s", ngwaddr);
				clrtoeol();
				refresh();

				curgets(nnmserver , 16, C_LINE, C_COL, nameserver, 0, nmserver, HELP_NSA, TYPE_IP);
				mvprintw(POS_NS, POS_COLNEW, "%s", nnmserver);
				clrtoeol();
				refresh();
				break;
				
			case ITEM_USE_NEW:
				if((fp = fopen(FILE_DEFAULTS, "w")) == NULL)
				{
					mvaddstr(1, (40 - strlen(nowrite) / 2), nowrite);
					break;
				}
				fprintf(fp, "%s\n", nipaddr);
				fprintf(fp, "%s\n", nsnmask);
				fprintf(fp, "%s\n", nifflags);
				fprintf(fp, "%s\n", nhname);
				fprintf(fp, "%s\n", ndname);
				fprintf(fp, "%s\n", ngwaddr);
				fprintf(fp, "%s\n", nnmserver);
				fclose(fp);
				finish();
				break;

			default:
				strcpy(nipaddr,   m_ipaddr[choice - ITEM_CUSTOM]);
				strcpy(nsnmask,   m_snmask[choice - ITEM_CUSTOM]);
				strcpy(nifflags,  m_ifflags[choice - ITEM_CUSTOM]);
				strcpy(ngwaddr,   m_gwaddr[choice - ITEM_CUSTOM]);
				strcpy(nhname,    m_hname[choice - ITEM_CUSTOM]);
				strcpy(ndname,    m_dname[choice - ITEM_CUSTOM]);
				strcpy(nnmserver, m_nmserver[choice - ITEM_CUSTOM]);

				mvprintw(POS_IP, POS_COLNEW, "%s", nipaddr);
				clrtoeol();
				mvprintw(POS_SN, POS_COLNEW, "%s", nsnmask);
				clrtoeol();
				mvprintw(POS_FL, POS_COLNEW, "%s", nifflags);
				clrtoeol();
				mvprintw(POS_GW, POS_COLNEW, "%s", ngwaddr);
				clrtoeol();
				mvprintw(POS_HN, POS_COLNEW, "%s", nhname);
				clrtoeol();
				mvprintw(POS_DN, POS_COLNEW, "%s", ndname);
				clrtoeol();
				mvprintw(POS_NS, POS_COLNEW, "%s", nnmserver);
				clrtoeol();
				break;
		}
	}
	return(0);
}

/*---------------------------------------------------------------------------*
 *	clean up and exit
 *---------------------------------------------------------------------------*/
static void
finish(void)
{
	move(23, 0);
	refresh();
	endwin();
	exit(0);
}

/*---------------------------------------------------------------------------*
 *	read menu defaults file
 *---------------------------------------------------------------------------*/
static int
rd_menudefs(void)
{
	FILE *fp;
	char buffer[81];
	int tab;
	int state;
	char *cp;

	if((fp = fopen(FILE_MENUDEFS, "r")) == NULL)
	{
		mvaddstr(1, (40 - strlen(nomdefs) / 2), nomdefs);
		return(0);
	}

	/* read timeout value */

	for(;;)
	{
		if((cp = fgets(buffer, 80, fp)) != NULL)
		{
			if(buffer[0] == '#')
				continue;
			else
				break;
		}
		else
		{
			mvaddstr(1, (40 - strlen(nomdefs) / 2), nomdefs);
			return(0);
		}
	}

	sscanf(buffer, "%d", &keytimeout);
	if(keytimeout < 0)
		keytimeout = 0;
	if(keytimeout > TIMEOUT_MAX)
		keytimeout = TIMEOUT_MAX;

	/* config tables contents */

	for(tab=0; tab < TABLEN; tab++)
	{
		for(state=ST_HEADL; state <= ST_NSERV; state++)
		{
			while((cp = fgets(buffer, 80, fp)) != NULL)
			{
				if(buffer[0] == '#')
					continue;
				else
					break;
			}

			if(cp == NULL)
			{
				fclose(fp);
				return(tab);
			}			

			switch(state)
			{
				case ST_HEADL:
					strcpy(m_menuitem[tab], buffer);
					m_menuitem[tab][strlen(buffer)-1] = '\0';
					break;
				case ST_IPADR:
					sscanf(buffer, "%s", m_ipaddr[tab]);
					break;
				case ST_SNMSK:
					sscanf(buffer, "%s", m_snmask[tab]);
					break;
				case ST_IFLGS:
					strcpy(m_ifflags[tab], buffer);
					m_ifflags[tab][strlen(buffer)-1] = '\0';
					break;
				case ST_HNAME:
					sscanf(buffer, "%s", m_hname[tab]);
					break;
				case ST_DNAME:
					sscanf(buffer, "%s", m_dname[tab]);
					break;
				case ST_GWADR:
					sscanf(buffer, "%s", m_gwaddr[tab]);
					break;
				case ST_NSERV:
					sscanf(buffer, "%s", m_nmserver[tab]);
					break;
			}
		}	
	}
	fclose(fp);
	return(tab);
}

/*---------------------------------------------------------------------------*
 *	print headerline
 *---------------------------------------------------------------------------*/
static void
hdrline(int row)
{
	char buf[80];
	
	int i = COLS-1;
	int j;
	
	move(row, 0);

	standout();
	attron(A_ALTCHARSET);
	for(j=5; j>0; j--, i--)
		addch(ACS_HLINE);
	attroff(A_ALTCHARSET);

	standout();
	sprintf(buf, " set network parameters ");
	i -= strlen(buf);
	addstr(buf);

	attron(A_ALTCHARSET);	
	for(j=20; j>0; j--, i--)
		addch(ACS_HLINE);
	attroff(A_ALTCHARSET);	

	standout();
	sprintf(buf, " version %s.%s (%s) ", VERS, REV, __DATE__);
	i -= strlen(buf);
	addstr(buf);

	attron(A_ALTCHARSET);	
	while (i--)
		addch(ACS_HLINE);
	attroff(A_ALTCHARSET);
	standend();
}

/* EOF */
